{ config, pkgs, lib, ... }: {

  imports = [

    ../../components
    ../../system/desktop
    ../../system/server/netdata.nix

    ./hardware-configuration.nix
    ./packages.nix
    ./syncthing.nix
    ./tinc.nix
    ./cups.nix

    #./wifi-access-point.nix
    #./wireshark.nix
    #./scanner.nix
    ./qemu.nix
    ./wireguard.nix

  ];


  components.gui.enable = true;

  services.nginx.enable = true;

  networking.hostName = "cream";

  system.custom.wifi.interfaces = [ "wlp166s0" ];

  security.wrappers = {
    pmount = {
      source = "${pkgs.pmount}/bin/pmount";
      setuid = true;
      owner = "root";
      group = "root";
    };
    pumount = {
      source = "${pkgs.pmount}/bin/pumount";
      setuid = true;
      owner = "root";
      group = "root";
    };
  };

  programs.custom.video.enable = false;
  programs.custom.steam.enable = true;
  services.printing.enable = true;

  # fonts
  # -----
  programs.custom.urxvt.fontSize = 16;
  programs.custom.urxvt.fontType = "vector";
  programs.custom.xterm.fontSize = 16;
  # todo : add xterm fontType
  # programs.custom.xterm.fontType = "vector";

  virtualisation = {
    docker.enable = true;
    podman.enable = true;
    virtualbox = {
      host.enable = false;
      guest.x11 = false;
      guest.enable = false;
    };
  };

  configuration.desktop = {
    width = 2256;
    height = 1504;
  };

  services.xserver.desktopManager.gnome.enable = true;
  #services.xserver.displayManager.lightdm.enable = false;
  #services.xserver.displayManager.sddm.enable = true;

  custom.samba-share = {
    enable = false;
    folders = {
      share = "/home/share";
      video = "/home/video-material";
    };
  };


  # for congress and streaming
  hardware.opengl = {
    enable = true;
  };

  system.stateVersion = "22.11";

}
